import multiprocessing as mp
import igraph
import itertools
import os
from numpy import *
from scipy import random
import time
import numpy as np
import gc
from collections import Counter
import sys

# Model parameters
GRAPHSIZE=8000
BETA=0.4
RUNTIMES=100
rseed = 1
paretoC = 1
SCALE = 1

location = sys.argv[1] # recommended full path to save folder
if len(sys.argv[2])>0:
	procnum = int(sys.argv[2])  # number of processes
else: 
	procnum = 4

def connection_calculator(Number, BETA, proc, paretoC): 
	"""
	Calculates the probability for every city pair connection
		"""
	random.seed()
	X = random.rand(Number)
	Y = random.rand(Number)
	W = (random.pareto(paretoC, Number)+1.)*SCALE
	
	W /= max(W)
	w = sqrt(W)
	
	w *= BETA # weighted city size w ~ \beta * sqrt(W)
	
	Number = len(X)
	g = igraph.Graph(directed=False)
	g.add_vertices(Number)
	coin = array(list(itertools.combinations(arange(Number),2))) # combinations of indexes 
	
	coco = array(list(itertools.combinations(column_stack((X,Y)),2))) # combinations of coordinates
	R12 = sqrt((coco[:,0,0]-coco[:,1,0])**2+(coco[:,0,1]-coco[:,1,1])**2) # distances of all combinations ( r_ij )

	WR_truth = w[coin[:,0]]+w[coin[:,1]]-R12 > 0
	
	# Using only giant component
	g.add_edges(coin[WR_truth])
	components = g.clusters(igraph.WEAK)
	f = components.giant()
	
	CS = components.sizes()
	CSMI = CS.index(max(CS)) #components sizes max index, giant components index in membership
	fdg = f.degree()

	# Recalculating the possible combinations, with the giant component
	dg = Counter(fdg)
	degree = array(dg.items())
	memb = array(components.membership)
	Xnew = X[where(memb==CSMI)]
	Ynew = Y[where(memb==CSMI)]
	Wnew = W[where(memb==CSMI)]
	Number2 = len(Xnew)

	coin2 = array(list(itertools.combinations(arange(Number2),2))) # combinations of indexes 

	coco2 = array(list(itertools.combinations(column_stack((Xnew,Ynew)),2))) # combinations of coordinates
	distances = sqrt((coco2[:,0,0]-coco2[:,1,0])**2+(coco2[:,0,1]-coco2[:,1,1])**2) # distances of all combinations ( r_ij )

	hops = array(f.shortest_paths())[triu_indices(f.vcount(),1)]
	
	# saving the files
	savetxt(location+str(Number)+"_hopVdist_"+str(BETA).replace(".","p")+"_"+str(paretoC).replace('.','p')+'_'+str(proc)+".dat",  column_stack((distances, hops)), delimiter=' ', fmt="%f") # writing hops and distances in correspondong dat files
	savetxt(location+str(Number)+"_degree_"+str(BETA).replace(".","p")+"_"+str(paretoC).replace('.','p')+'_'+str(proc)+".dat", degree, delimiter=' ', fmt="%d")
	
# starting parallel runs
def run_sim(j):
	random.seed()
	connection_calculator(GRAPHSIZE, BETA, j, paretoC)
	gc.collect()

pool = mp.Pool(procnum)
pool.map(run_sim, range(RUNTIMES))



